/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.monitoring;

import java.util.LinkedList;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.wsrf.ResourceKey;

public class EventDispatchQueue
{
    /** Log4J logger */
    private static Log logger
        = LogFactory.getLog(EventDispatchQueue.class);
    private final int THREAD_COUNT = 1;
    private LinkedList cachedEvents = null;
    private JobStateMonitor monitor = null;
    private boolean initialized = false;

    public EventDispatchQueue (
           JobStateMonitor        monitor)
    {
        this.cachedEvents = new LinkedList();
        this.monitor = monitor;
        this.initialized = false;
    }

    private synchronized void initialize ()
    {
        // start all run queue instances
        // 1 thread is enough
        for (int index=0; index<THREAD_COUNT; index++)
        {
            EventDispatchThread dispatchThread = new EventDispatchThread(
                "EventDispatchThread_" + index,
                this);
            dispatchThread.setDaemon(true);
            dispatchThread.start();
        }
        initialized = true;
    }

    public synchronized void add (SchedulerEvent event)
    {
        if (!initialized) {
           initialize();
        } 

        cachedEvents.add(event);
        notify();
    }

    public synchronized SchedulerEvent remove()
    {
        return (SchedulerEvent) cachedEvents.removeFirst();
    }

    public synchronized int size()
    {
        return cachedEvents.size(); 
    }

    synchronized JobStateMonitor getMonitor ()
    {
        return this.monitor;
    } 

}
