/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.monitoring;

import org.globus.wsrf.ResourceKey;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class EventDispatchThread extends Thread
{

    /** Log4J logger */
    private static Log logger = LogFactory.getLog(EventDispatchThread.class);
    private boolean done = false;
    private EventDispatchQueue dispatchQueue = null;
    private JobStateMonitor monitor = null;
    private JobStateChangeListener listener = null;

    EventDispatchThread (String                 name,
                         EventDispatchQueue     queue)
    {
        super(name);
        this.dispatchQueue = queue;
        this.monitor = queue.getMonitor();
        this.listener = queue.getMonitor().getListener();
    }

    public synchronized void terminate()
    {
        this.done = true;
    }

    public void run()
    {
        while (!this.done)
        {
            SchedulerEvent event;

            synchronized (this.dispatchQueue)
            {
                while ((!this.done) && (dispatchQueue.size() == 0))
                {
                    try
                    {
                        dispatchQueue.wait();
                    }
                    catch (InterruptedException e) { }
                }
            }

            do
            {
                synchronized (dispatchQueue)
                {
                    if (dispatchQueue.size() > 0)
                    {
                        event = (SchedulerEvent) dispatchQueue.remove();
                        logger.debug("DispatchQueueSize: "+
                                     dispatchQueue.size());
                    }
                    else
                    {
                        event = null;
                    }
                }

                if (event != null)
                {
                    ResourceKey resourceKey =
                             monitor.getMapping(
                                event.getLocalId());

                    // only if the resource has not been unregistered
                    // in the meantime
                    if (resourceKey != null) {
                             listener.jobStateChanged(
                                  resourceKey,
                                  event.getLocalId(),
                                  event.getTimeStamp(),
                                  event.getState(),
                                  event.getExitCode());

                       logger.debug("Job "+resourceKey.getValue()+
                                    ": EventQueueThread: dispatched event "+
                                    event.getState());
                    }

                }
            }
            while ((!done) && (event != null));
        }
    }
}
